from math import sin, cos, sqrt, asin, radians
from data import data_header, data_as_list_of_lists

###### 1. Import the dataset #######
# This imports the function `data_as_list_of_lists` from `data.py`
# This only works if `data.py` is in this folder
from data import data_header, data_as_list_of_lists

# Load the header and the dataset
bluebike_header = data_header("bluebike-5561.csv")
bluebike_data = data_as_list_of_lists("bluebike-5561.csv")

# Like last week, this data is in the form of a list of lists.
print(type(bluebike_data))
print(type(bluebike_data[0]))

# Like last week, it will be helpful to inspect some of this data.
# You should print out the header, and then every trip record in the data:
print(bluebike_header)
for trip in bluebike_data:
    print(trip)

###### 2. Distances between stations and lengths of trips ########

# We will be interested in how far a bike has traveled.
# Sadly, we don't know the exact routes of the bikes; the best approximation
# we can make is to calculate the distance between the station the bike originated
# at the station it was left at. For that we have already defined a formula to
# approximate this distance using the Haversine formula:

def distance(loc1, loc2):
    """
    Calculate the distance between two locations using the Haversine formula.

    Parameters
    ----------

        loc1, loc2 (2-tuple of floats)
           Locations encoded as (latitude, longitude) pairs

    Returns
    --------

        dist (float)
            The distance between the two locations, in miles

    """

    # tuples can be *destructured* and assigned to new variables
    # >>> a, b = ("x", "y")
    # >>> a == "x"
    # True

    lat1, long1 = loc1
    lat2, long2 = loc2

    delta_lat = radians(lat2 - lat1)
    delta_long = radians(long2 - long1)

    lat1 = radians(lat1)
    lat2 = radians(lat2)

    RADIUS = 3959.87433
    a = sin(delta_lat / 2)**2 + cos(lat1) * cos(lat2) * sin(delta_long / 2)**2

    return RADIUS * 2 * asin(sqrt(a))

# We would like to calculate the distance this bike traveled on each trip.
# We would also like to store this value in our dataset, so that we don't have
# to recalculate it later.
#
# Use a for loop to calculate and store the distance for each record:
for trip in bluebike_data:
    trip.append(distance(trip[3],trip[4]))

# Update the header to include the new column, as well:
bluebike_header.append('trip_distance')
print(bluebike_header)

"""

Do the values you are getting make sense? What are you expecting?

I'm expecting values in miles. The city is fairly small, so at most
a few miles.

"""

###### 3. From distance to speed #######

# First, write a function to get the average speed the bike traveled
# in a trip, in miles per hour. Because trip_duration is in seconds,
# you will need to convert the duration to hours yourself:

def speed(distance, duration):
    """
    Given a starting and ending location (in latitude/longitude coordinate pairs)
    and the length of a trip, return the average speed traveled during the trip,
    in miles per hour.

    Parameters
    ----------
        distance (float)
           The distance covered by a trip, in miles

        duration (int)
           The duration of the trip, in seconds

    Returns
    -------
        speed (float)
           The average speed the bike was traveling during the trip,
           in miles per hour
    """
    # replace pass with your code!
    duration_hours = duration/60/60
    speed = distance/duration_hours
    return speed

# We are interested in the *range* of speeds that people using this bike keep.
# Use a for loop with if statements to update the minimum and maximum speed
# of trips in the dataset:
min_speed = 10000000
max_speed = 0

for trip in bluebike_data:
    trip_speed = speed(trip[9],trip[5])
    if trip_speed > max_speed:
        max_speed = trip_speed
    if trip_speed < min_speed:
        min_speed = trip_speed

print("min_speed:",min_speed)
print("max_speed:",max_speed)

# Next, find the average speed over all trips taken by this bike:
sum_speed = 0
num_trips = 0
for trip in bluebike_data:
    trip_speed = speed(trip[9],trip[5])
    if trip_speed > 0: # optional
        sum_speed += trip_speed
        num_trips += 1

print("avg_speed:",sum_speed/num_trips)


"""
In addition to printing out the values, record the minimum, maximum, and average
speed here.

min speed: 0.0 mph
max speed: 12.01 mph
average speed: 5.54 mph or 5.60 mph

Are these values reasonable? Does it seem right to include all trips when calculating
the average speed?

Blue Bikes that are returned to the same dock will go a distance of zero, which
is clearly not measuring what we think it's measuring.

One argument is that a speed of zero is an artifact of the way the data is collected,
so we should probably exclude those trips from calculations of average speed.

Also acceptable are good arguments to keep those values in. For instance, noting
that a mismatch between the distance we calculate and the distance traveled is a
problem in all the trips, not just the zeros. Better to keep them in and be clear.

Finally, students could note that it doesn't make a big difference.

"""

# Now, we want to be able to apply the same logic across subsets of our dataset,
# or on different bikes/months. Write a function that takes a dataset (a list of
# lists with the same column structure as the dataset we have been using) and
# returns the average speed for all trips in the dataset:

def average_speed(dataset):
    """
    Given a starting and ending location (in latitude/longitude coordinate pairs)
    and the length of a trip, return the average speed traveled during the trip,
    in miles per hour.

    Parameters
    ----------
        dataset (list of lists)
            A list of records of trips taken by Bluebikes.

    Returns
    -------
        avg_speed (float)
           The average speed across all records of trips in the dataset
    """
    # Replace pass with your code!
    sum_speed = 0
    num_trips = 0
    for trip in dataset:
        trip_speed = speed(trip[9],trip[5])
        if trip_speed > 0: # optional
            sum_speed += trip_speed
            num_trips += 1

    return sum_speed/num_trips

# Use the average_speed function on the full dataset to confirm it works.
# Hint: average_speed(bluebike_data) should produce the same number you calculated before.

print("avg_speed function:",average_speed(bluebike_data))

###### 4. Considering commuters and casuals ######

# Similar to last week, we're going to create subsets of the dataset.

subscribers = list()
customers = list()

# Use a combination of a for loop, and if/else statement, and the append method,
# create the subsets of subscribers and customers based on the user_type column.
for trip in bluebike_data:
    if trip[6] == "Subscriber":
        subscribers.append(trip)
    if trip[6] == "Customer":
        customers.append(trip)

# Now, use the average_speed function above to calculate the average speed of the
# subsetted datasets.

print("avg_speed subscribers:",average_speed(subscribers))
print("avg_speed customers:",average_speed(customers))

"""
In addition to printing the average speed, write the answers here.

Subscribers: 5.84 or 5.94 mph
Customers: 4.06 or 4.06 mph

Note the title of the section in the handout. What do these quantities tell us about
the differences between commuters and casual users of Blue Bikes?

Some discussion of the difference between what we want these categories to represent
what they actually *are*. But the result conforms to our expectations that those who
rely on Blue Bikes often, in the sense that they have a subscription, get where
they are going faster than those who do not. Quite a bit, even!

"""

##### 5. Extra stuff #######

### Column indexing
from statistics import median
def get_column(data, idx):
    """
    Given a dataset (in list of lists form) and and index number,
    return the indexed "column" of the dataset, i.e., the ith element
    of each record.

    Parameters
    ----------
        dataset (list of lists)
            A list of records of trips taken by Bluebikes.

        idx (integer)
            The desired element of each record

    Returns
    -------
        column (list)
           A list containing that element of each record
    """
    column = list()
    
    for row in data:
        column.append(row[idx])
    
    return column

print(median(get_column(bluebike_data, 7)))

### Optional function parameters
def average_speed(dataset, skip_loops=True):
    """
    Given a starting and ending location (in latitude/longitude coordinate pairs)
    and the length of a trip, return the average speed traveled during the trip,
    in miles per hour.

    Parameters
    ----------
        dataset (list of lists)
            A list of records of trips taken by Bluebikes.

        skip_loops (bool)
            Whether or not to ignore trips originating and arriving at the
            same station.

    Returns
    -------
        avg_speed (float)
           The average speed across all records of trips in the dataset
    """
    # Replace pass with your code!
    sum_speed = 0
    num_trips = 0
    for trip in dataset:
        trip_speed = speed(trip[9],trip[5])
        if trip_speed > 0:
            sum_speed += trip_speed
            num_trips += 1
        elif skip_loops:
                sum_speed += trip_speed
                num_trips += 1

    return sum_speed/num_trips

print(average_speed(bluebike_data))
print(average_speed(bluebike_data, skip_loops=False))
#### Boston Blue Bike #?????