import pandas as pd
from math import sin, cos, sqrt, asin, radians


def load_data(filename):
    """
    Take the Bluebikes data, or a subset of, and rename
    and collapse some columns, then return the updated
    data frame to be transformed into a non-data-frame.
    """
    df = pd.read_csv(filename, sep=",")

    df["start_location"] = list(
        zip(df["start station latitude"], df["start station longitude"])
    )
    df["end_location"] = list(
        zip(df["end station latitude"], df["end station longitude"])
    )

    df = df.rename(
        columns={
            "tripduration": "trip_duration",
            "starttime": "start_time",
            "stoptime": "end_time",
            "usertype": "user_type",
            "birth year": "birth_year",
            "start station name": "start_name",
            "end station name": "end_name",
        }
    )

    df = df.drop(
        columns=[
            "start station latitude",
            "start station longitude",
            "end station latitude",
            "end station latitude",
        ]
    )

    return df[
        [
            "bikeid",
            "start_name",
            "end_name",
            "start_location",
            "end_location",
            "trip_duration",
            "user_type",
            "birth_year",
            "gender",
        ]
    ]


def data_as_list_of_lists(filename):
    return load_data(filename).values.tolist()


def data_header(filename):
    return load_data(filename).columns.tolist()


def data_from_file(filename, bikeids):
    """
    Create a subset of the data, filtering on bike ids.
    """
    if isinstance(bikeids, int):
        bikeids = [bikeids]
    all_bikes = pd.read_csv(filename, sep=",")
    subset = all_bikes[all_bikes.bikeid.isin(bikeids)]

    return subset


######## This function runs when we call `python data.py` #########
# To run this, you must first download the original data file. It is available here:
# https://www.bluebikes.com/system-data

if __name__ == "__main__":
    subset = data_from_file("201908-bluebikes-tripdata", 5561)
    subset.to_csv("bluebike-5561.csv", sep=",", index=False)
